      subroutine dray(psival,rayrz,drayrz)
c
c     DRAY provides NAG integrator D02BBF with the ray derivatives
c     using the relation:         d(rmaj,zed)/dpsi=
c           [dpsi/d(rmaj,zed)]/([dpsi/drmaj]**2+[dpsi/dzed]**2),
c     where psi, a function of rmaj and zed, is:
c                psi=r/[r+rminor*(1.-r/Rwall)]
c     where r**2=(rmaj-Rmajor)**2+zed**2 and Rwall=Rwall(theta)
c     where tan(theta)=zed/(rmaj-Rmajor).  Decreasing Rminor
c     results in a grid which is denser toward the center.
c
c Input:
c
c psival (ignored)
c rayrz(1) = R	  current major radius
c rayrz(1) = Z	  current height above midplane
c
c Output:
c
c drayrz(1) = d R / d Psi
c drayrz(2) = d Z / d Psi
c
c Integrating along drayrz will map out a coordinate of constant
c chi.
c
c GWH 22-Mar-1991:  rewrote this subroutine to work in (dR,Z)
c coordinates rather than (R,Z) coordinates, where dR=R-Rmajor.
c This was done to reduce round-off errors near the origin where
c R-Rmajor is needed.  This greatly improved the grid for
c highly-shaped tokamaks like CIT.
 
      implicit none
      include 'SPRINC'
      real psival,rayrz(2),drayrz(2)
 
      include 'SPRUCOM'
 
      integer ichirz
      real origin(nchi)
      common /ray/ ichirz,origin
 
c local variables:
      real pi,spline,dpline,sprime,dprime,r,th,sinth,costh,dth,
     %	  s,s0,s1,rw,drw,wrk,draydr,draydt,grad
      integer itest,itstp1
 
      data pi /3.14159265/
 
c     cubic spline statement functions
      spline(s)=s*s*(2.*s-3.)+1.
      dpline(s)=s*(s*(s-2.)+1.)
      sprime(s)=6.*s*(s-1.)
      dprime(s)=s*(3.*s-4.)+1.
 
c
c     Compute r,sinth,costh,rwall(thetha),and drwall/dthetha
      r=sqrt(rayrz(1)**2+rayrz(2)**2)
      if (r.le. draytol) then
      sinth=sin(origin(ichirz))
      costh=cos(origin(ichirz))
      else
      sinth=rayrz(2)/r
      costh=rayrz(1)/r
      endif
      th=atan2(sinth,costh)
      if (th.lt.0.) th=th+2.*pi
 
c
c interpolate in theta with a cubic spline
c to find rwall and d(rwall)/dtheta:
c
      dth=theta(2)-theta(1)
      itest=(th/dth)+1
      itstp1=itest+1
 
      s0=(th-theta(itest))/dth
      s1=(theta(itstp1)-th)/dth
      rw=radius(itest)*spline(s0)+radius(itstp1)*spline(s1)+
     %   dth*(drdth(itest)*dpline(s0)-drdth(itstp1)*dpline(s1))
      drw=(radius(itest)*sprime(s0)-radius(itstp1)*sprime(s1))/dth+
     %    drdth(itest)*dprime(s0)+drdth(itstp1)*dprime(s1)
c
c     Compute d(rmaj,zed)/dpsi
c
c draydr = d Psi / d r
c draydt = (1/r)  d Psi / d theta
c
c drayrz(1) = d R / d Psi
c drayrz(2) = d Z / d Psi
c
      wrk=1./(r+rminor*(1.-r/rw))**2
      draydr=rminor*wrk
      draydt=-rminor*r*drw*wrk/rw**2
 
c try flux surfaces which approach circularity faster as the
c origin is approached:
      draydr=1./rminor-3*r**2/rminor/rw**2+3*r**2/rw**3
      draydt=(2*r**3/rminor/rw**3-3*r**3/rw**4)*drw
 
c try flux-surfaces which have same shape near origin as at edge:
c      draydr=1./rw
c      draydt=-drw/rw**2
c There may be fundamental reasons why this doesn't work.
c An ORTHOGONAL coordinate system may have to be circular near
c the axis.
 
      grad=1./(draydr**2+draydt**2)
      drayrz(1)=(costh*draydr-sinth*draydt)*grad
      drayrz(2)=(sinth*draydr+costh*draydt)*grad
c
      return
      end
