	subroutine fpcross
c
c calculates the explicit fluxes due to the cross-derivative terms
c in the Fokker-Planck equation, including an algorithm for
c limiting the fluxes so that f>0 is (approximately) maintained.
c
c this has been split off from subroutine IMP to allow it to
c be called by MOMS as well to calculate the initial power.
c
c 7-Aug-1989:
C A number of algorithms for the cross-derivative terms have been
C tried.  However, the clear winner is the FLUX-LIMITED TRANSPORT
C algorithm, where we limit the explicit cross-derivative flux to
C guarantee that f>0 (approximately).  Because we use centered
C differencing, this method is second order accurate in most places,
C except where the flux limiter has to be applied.  This algorithm is
C very robust for for large time steps, yet still guarantee's
C convergence to the proper solution as the time step is made small.
C It also converges quicker than the other algorithms as the time step
C is made small.
c
C My FLUX-LIMITED TRANSPORT algorithm is vaguely related to Borris's
C FLUX-CORRECTED TRANSPORT algorithm :
c
c Borris and Book, Methods in Computational Physics, Vol. 16, 1976,
c or Steven T. Zalesak, "Fully Multidimensional Flux-Corrected Transport
c algorithms for fluids," JCP V.31 p. 335 (1979))
c
c and appears to be conceptually similar to:
c
C Lawrence D. Cloutman, "A Convective Flux Limiter for Non-Lagrangian
c Computational Fluid Dynamics," JCP V. 73, p. 349 (1987).
c
C My flux limiter only prevents f<0 while Cloutman's limiter is
C stricter and prevents the formation of any new minima or maxima.
c
c I have tried a number of other algorithms for calculating the
c cross-derivative terms.  For example, I tried a minimum gradient
c algorithm, where the minimum possible value of df/dxsi was using
c in calculating the cross-derivative flux in the energy direction
C (there is some leeway in calculating this term, because to
C calculate the energy flux due to df/dxsi at (i+.5,j), one
C can use any combination of the 4 non-centered gradients involving the
C 6 points closest to (i+.5,j)).
C This was tried to mimic donor cell differencing.  While it
c seemed to help some, the flux-limited algorithm described above is
C much better.
C
C
C     COMMON BLOCKS
C
	include 'FPPCOM'
 
c	data zcross /1.0/   ! weighting of centered cross-derivatives
c (centered differencing is now the default, unless the flux-limiter
c kicks on.)
 
c--the constants jm and jp are frequently used because f is
c       symmetric in xsi in the trapped region:
c       f(jm-i)=f(jp+i) for i=0 to njTrap-1
c       jm and jp are the arbitrary grid equivalents of 26 and 27,
c       the magic numbers which Rob used frequently in his original
c       code.
	parameter (jm=maxxsi/2, jp=maxxsi/2+1)
 
c--implement conservative or df/dE=0 B.C.'s:
	ibotto=0
	if(LLwbnd .eq. 0)ibotto=1
 
	do 300 j=1,maxxsi
	do 200 i=ibotto,maxen
 
	if(i .eq. maxen) goto 170   !no flux out the top
 
c df/dxsi plus:
 
	zf1=(f1(i,j-1)+f1(i+1,j-1))
c	zf2=(f1(i,j)+f1(i+1,j))
	zf3=(f1(i,j+1)+f1(i+1,j+1))
c
c	zg2=zf2-zf1
c	zg3=zf3-zf2
c	zwgt=abs(zg2)+abs(zg3)
c	if(zwgt .gt. 0.0)then
c	    zd=(zg2*abs(zg3)+zg3*abs(zg2))/zwgt
c	else
c	    zd=0.0
c	endif
c	zd=zcross*(zf3-zf1)/2+(1.0-zcross)*zd
	zd=(zf3-zf1)/2
 
	fpfluxe(i,j)=-dec(i,j)*zd
 
170	if(j .eq. maxxsi) goto 200  !no flux out the side
 
c df/de plus:
 
	zf1=(f1(i-1,j)+f1(i-1,j+1))
c	zf2=(f1(i,j)+f1(i,j+1))
	zf3=(f1(i+1,j)+f1(i+1,j+1))
 
c	zg2=zf2-zf1
c	zg3=zf3-zf2
c	zwgt=abs(zg2)+abs(zg3)
c	if(zwgt .gt. 0.0)then
c	    zd=(zg2*abs(zg3)+zg3*abs(zg2))/zwgt
c	else
c	    zd=0.0
c	endif
c	zd=zcross*(zf3-zf1)/2+(1.0-zcross)*zd
	zd=(zf3-zf1)/2
 
	fpfluxa(i,j)=-dac(i,j)*zd
 
200     continue
300     continue
 
c impose a flux limiter:
 
c--implement conservative or df/dE=0 B.C.'s:
	ibotto=0
	if(LLwbnd .eq. 0)ibotto=2
 
	do 500 j=1,maxxsi
	do 400 i=ibotto,maxen
 
c sum of all the fluxes out of cell (i,j):
	zfluxout=max(0.,fpfluxe(i,j)) - min(0.,fpfluxe(i-1,j))
	1   + max(0.,fpfluxa(i,j)) - min(0.,fpfluxa(i,j-1))
 
c which cannot exceed the total number of particles in cell (i,j):
	ztest=f1(i,j)/fpdtjac(i,j)
 
	zrat=1.
	if(zfluxout .gt. 0.0) zrat=min(1.,ztest/zfluxout)
 
c if we want to vectorize this on the cray and have to remove the
c if statements, I know we can replace the if statements below
c in terms of SIGN's, but I don't know how to replace the above
c if statement without having problems with the case ztest=0.0!
 
c limit the outgoing fluxes to preserve f>0:
	if(fpfluxe(i,j) .gt. 0.0) fpfluxe(i,j)=fpfluxe(i,j)*zrat
	if(fpfluxe(i-1,j) .lt. 0.0) fpfluxe(i-1,j)=fpfluxe(i-1,j)*zrat
	if(fpfluxa(i,j) .gt. 0.0) fpfluxa(i,j)=fpfluxa(i,j)*zrat
	if(fpfluxa(i,j-1) .lt. 0.0) fpfluxa(i,j-1)=fpfluxa(i,j-1)*zrat
 
400     continue
500     continue
 
	return
	end
