	subroutine fpoper
c--This routine sets up the coefficients of an arbitrary
c       Fokker Planck equation written in conservative form:
c
c df/dt = (1./qjac) ( d/dE ( dea f + deb df/dE + dec df/d(xsi) )
c	      + d/d(xsi) ( dab f + dab df/d(xsi) + dac df/dE ) )
c
c--The discrete version of the general F.P. equation is given
c       in "Particle Conserving B.C.'s for Fpp", Oct. 17, 1983, gwh,
c       which is based on McCoy, Mirin, &Killeen, "FPPAC: A
c       Two-Dimensional Multispecies Nonlinear Fokker-Planck
c       Package", Computer Physics Communications, V.24, p. 37 (1981).
c
c--A few notes are in order here.  In this code, qjac has had 2 pi q R
c       factored out of it, so that qjac = sqrt(enerev) abs(xsi) btime.
c       Because f must be symmetric in the trapped region, we must
c       also have dea, deb, & dab symmetric in the trapped region
c       while dec, daa, & dac must be anti-symmetric.
c       Outside the trapped region, there is no symmetry requirement.
c       it is assumed in PREPRE that the off-diagonal terms
c       dec and dac vanish at the left and right boundaries
c       (xsi = +/- 1.0) and at the bottom boundary (enerev=0).
c
c--This subroutine evaluates the pitch angle scattering terms in the
c       Fokker-Planck equation.  RFOPER evaluates the RF terms, ELCF
c       finds the ohmic electric field terms.  The energy drag and
c       diffusion terms are added in PREADI.
c
c       Rob's original version of FPP did not use a conservative
c       form of the pitch angle scattering operator,
c	(1/J) d/dx( B df/dx).  Instead he expanded it as
c	(B/J) d2f/dx2 + (dB/dx/J) df/dx.
c       The number conserving coefficients have logarithmic
c       singularities at the trapped passing boundary which cancel
c	each other in the long run. The advantage of Rob's approach is
c	that the coefficients stay finite everywhere, although it is no
c	longer written as the divergence of a flux and so does not
c	conserve particles when differenced.
c
 
	include 'FPPCOM'
 
 
c--initialize everything to zero:
	do 500 i=-1,maxen+1
	do 500 j=0,maxxsi+1
	dea(i,j)=0.0
	deb(i,j)=0.0
	dec(i,j)=0.0
	daa(i,j)=0.0
	dab(i,j)=0.0
	dac(i,j)=0.0
 
	fpfluxe(i,j)=0.0
	fpfluxa(i,j)=0.0
 
	if(i .eq. -1
	1       .or. i .eq. maxen+1
	1       .or. j .eq. 0
	1       .or. j .eq. maxxsi+1) go to 500
 
c--infinite P.A.S. at origin:
	if(i .eq. 0)goto 500
 
	if(j .eq. maxxsi .or. j .eq. maxxsi/2)goto 500
c--the reason for this if statement is two-fold:
c--The DAB coefficient is evaluated between grid points.
c       dab(i,j) is really dab(i,j+.5), so j=maxxsi is off
c       the grid.  Also dab goes to zero at xsi=0 (j=maxxsi/2).
c
	avxsi=(xsi(j+1)+xsi(j))/2
	avxsi2=(xsi(j+1)**2+xsi(j)**2)/2
	avbtim=(btime(j+1)+btime(j))/2
	avbrat=(bratio(j+1)+bratio(j))/2
	dab(i,j)=scat(i)*avbtim*(avbrat+avxsi2)
	1       *enerev(i)**.5*(1-avxsi2)/abs(avxsi)
 
c       write(lsysout,*) i,j,dab(i,j)
 
C************************************************************
	if(dab(i,j) .lt. 0.0)then
	write(lsysout,*) 'error in fpoper, dab<0.0'
	write(lsysout,*) 'i,j,avxsi,avxsi**2,avbrat,dab(i,j):'
	write(lsysout,*) i,j,avxsi,avxsi**2,avbrat,dab(i,j)
	write(lsysout,*) ' will set dab=0.0.'
	dab(i,j)=0.0
	endif
c*********************************************************
 
500     continue
 
	return
	end
