	subroutine imp
c advance the Fokker-Planck equation in time using implicit operator
c splitting for all terms except the cross-derivative terms, which must
c be treated explicitly.
c
c Cross-derivative terms cause the old ADI method to be unstable unless
c the time-step is very small.  The new implicit operator splitting
c method has been found to be fairly stable.  For constant coeficients,
c it can be shown to be unconditionally stable.  In practice, it appears
c that the algorithm does go unstable if the time step is made too big,
c probably because the coefficients are not constant. The improved
c stability of the splitting method over the ADI method make it prefered,
c at least for ICRF heating where there are cross- derivatives.  Of
c course, stability does not imply accuracy, and it is a good idea to
c check that the time and grid step sizes are small enough to give
c accurate results by running the code with varying step sizes.
c
c 23-Feb-1989. GWH: Check out Bob Mccann's upgrade which allows IMP
c to vectorize on a CRAY computer.
c
c Oct. 14, 1985, gwh : changed cross-derivative terms to be
c evaluated between gridpoints.
c
c March 17, 1985 -- Put CX source before the implicit energy step
c (it had been after the implicit energy step).  This is the way
c it should be for proper implicit time advancement, and should
c clear up a problem I was having with the power to the ions.  If you
c are going to convert this routine to ADI, you have to add the CX
c source a second time to the code.  Probably just before the
c explicit energy time step.  GWH.
c
c       Implemented loss regions as implicit boundary conditions
c       on the implicit time steps.  This is a little more rigorous
c       thans Rob's method, but his method is equivalent for loss
c       regions with simple shapes.  See my notes in "Particle
c       Conserving B.C.'s for FPP" Oct. 17, 1983.  Also changed
c       fishbone losses to be turned on just once at the end, rather
c       than four times. GWH 28-SEP-1984.
c
C       UPDATED TO ALLOW LOSS REGIONS TO DEPEND ON PITCH ANGLE
C       R.J.G. 10/3/81
C
C       UPDATED TO ALLOW INCLUSION OF A FISH-BONE LOSS MODEL
C       ROB GOLDSTON -- MAY 3, 1982
C
C       UPDATED TO ALLOW PARTICLE-CONSERVING LOWER BOUNDARY CONDITION
C       ROB GOLDSTON -- JUNE 15, 1983
C
c       improved particle-conserving lower boundary conditions
c       greg hammett -- oct. 4, 1983
c
c       speeded up by taking common factors out of do loops
c       greg hammett -- oct. 7, 1983
c
c       explicit treatment of mixed derivative terms added in order
c       to include a Quasilinear RF operator.
c       greg hamett -- oct. 23, 1983
c
c       generalized left and right boundary conditions (xsi= +/-1)
c       for arbitrary F.P. operator.  This helped improve stability
c       a little.
c       greg hammett -- Nov. 1, 1983
c
c       only force f to be non-negative at the end, after the whole
c       operator has had a chance to smooth things out.
c       greg hammett  Nov. 1, 1983.
c
c       Generalized to arbitrary grid. gwh Dec. 20, 1983
c
c       As early as Jan. 31, 1984 I tried Saul'yev type semi-implicit
c       schemes to try to speed up the code with an unconditionally
c       stable method.  However, some of the methods I tried turned
c       out to be unconditionally unstable.  It was not until
c       July 1984 that I found a method which would work.
c
C Other algorithms which have been tried include:
c
C McKee and Mitchell's ADI method for mixed-derivatives (McKee and
C Mitchell, The Computer Journal, V. 13., p. 81 (1970), and Mitchell
C and Griffiths, The Finite Difference Method in Partial Differential
C Equations, p. 81-82 1980).   In ways, this is similiar to the
C semi-implicit schemes advertised by Harned and Schank for MHD
C problems. This algorithm is only marginally stable (like
C Cranck-Nickolson half-implicit and half-explicit schemes) for
C constant coefficients, and is found emperically to fail for our
C problem.
c
c I have also tried a modified McKee-Mitchell algorithm,
c where I do the energy and pitch angle steps fully implicitly,
c but I add in an explicit energy step at the beginning and subtract
c it out again later.  zfemult was the multiplier on the explicit
c energy step.  zfemult=1.0 was found to be frequently unstable,
c while zfemult=0.5 was found to be occasionally helpful when
c using unlimited centered or minimized differencing of the
c cross-derivative.
C
C Another possible set of algorithms, which I have not yet tried are
c the Hopscotch methods (p. 77-82 of Mitchell and Griffiths book (1980)).
C At first I thought it was not a particle
C conserving algorithm, but now I think it can be made conservative.
c
c 7-Aug-1989:
c A number of algorithms for the cross-derivative terms have been tried.
c However, the clear winner is the FLUX-LIMITED TRANSPORT algorithm,
c where we limit the explicit cross-derivative flux to guarantee
c that f>0.  Because we use centered differencing, this method is
c second order accurate in most places, except where the flux
c limiter has to be applied.  This algorithm is very robust for
c for large time steps, yet still guarantee's convergence to the proper
c solution as the time step is made small.  It also converges quicker
c than the other algorithms as the time step is made small.
c
C My FLUX-LIMITED TRANSPORT algorithm is vaguely related to Borris's
C FLUX-CORRECTED TRANSPORT algorithm (Borris and Book, Methods in
C Computational Physics, Vol. 16, 1976), and appears to be conceptually
C similar to:
c
C Lawrence D. Cloutman, "A Convective Flux Limiter for Non-Lagrangian
c Computational Fluid Dynamics," JCP V. 73, p. 349 (1987).
c
C My flux limiter only prevents f<0 while Cloutman's limiter is
C stricter and prevents the formation of any new minima or maxima.
c
c I have tried a number of other algorithms for calculating the
c cross-derivative terms.  For example, I tried a minimum gradient
c algorithm, where the minimum possible value of df/dxsi was using
c in calculating the cross-derivative flux in the energy direction
C (there is some leeway in calculating this term, because to
C calculate the energy flux due to df/dxsi at (i+.5,j), one
C can use any combination of the 4 non-centered gradients involving the
C 6 points closest to (i+.5,j)).
C This was tried to mimic donor cell differencing.  While it
c seemed to help some, the flux-limited algorithm described above is
C much better.
C
C
C 29-Mar-1989:  GWH.  Made a number of changes which improved the
C stability of the code.  We already had the top boundary conditions
C DEA(maxen+.5)=DEB(maxen+.5)=DEC(maxen+.5)=0.0.  I added the condition
C DAC(maxen,j)=0, and removed the old condition DAB(maxen,j)=0.  When
C thinking about boundary conditions at the top, it is useful to think
C of the point at maxen+1 as existing, but with an energy E(maxen+1)
C that goes to infinity and f(maxen+1) which goes to zero.
C
C     COMMON BLOCKS
C
	include 'FPPCOM'
 
c	data zfemult /0.0/  ! 1.0=McKee-Mitchell ADI, 0.0=implicit splitting
c	real zfe(-1:maxen+1,0:maxxsi+1)  ! zfe dimensioned like f2
 
	double precision sum,averag	    !@VAX
C@       REAL SUM,AVERAG                    !CRAY
	double precision zad,zae,zb
C
c--de and da are local arrays which are used in inverting
c       the tri-diagonal matrix which arises when using the ADI
c       method.
c--gwh 11/27/84: replaced
        REAL    de(1:maxxsi,0:maxen)  !RTM - NOTE CHANGE OF SUBSCRIPT ORDER
	REAL	da(0:maxen,1:maxxsi)  !RTM
c with this:
C	real de(0:maxen),da(1:maxxsi)	!RTM
c (23-Fec-1989, RTM changed them back again).
c and made associated changes in the program below.  (the extra
c dimension was irrelevant.  This change only resulted in a 1%
c improvement in speed, and only 20 Kbytes of memory. Not a big
c difference.  Oh well...)
 
c--the constants jm and jp are frequently used because f is
c       symmetric in xsi in the trapped region:
c       f(jm-i)=f(jp+i) for i=0 to njTrap-1
c       jm and jp are the arbitrary grid equivalents of 26 and 27,
c       the magic numbers which Rob used frequently in his original
c       code.
	parameter (jm=maxxsi/2, jp=maxxsi/2+1)
 
C********************************************************************
C								    *
C       ENERGY SPACE EXPLICITLY				            *
C								    *
C********************************************************************
 
c evaluate the fluxes due to the cross-derivative terms, including
c flux limiters to preserve f>0:
	if(prftot .le. 0.0) go to 1052
	call fpcross
 
	ibotto=0
	if(LLwbnd .eq. 0)ibotto=2
 
	do 325 j=1,maxxsi
c--save some time by skipping over the symmetric half of trapped space
c--another way to save some time would be to restrict the calculation
c       to the confined part of velocity space (loss .ne. 0).
	IF(J .gt. jm .AND. j .lt. jm+njTrap) go to 325
 
	do 320 i=ibotto,maxen
 
! McKee-Mitchell does an explicit energy step now, and subtracts it
! back out after the implicit pitch angle step and before the
! implicit energy step:
c	zfe(i,j)=zfemult*(f1(i,j)*(1-bete(i,j))-alphe(i,j)*f1(i+1,j)
c	1   -game(i,j)*f1(i-1,j))
 
c	f2(i,j)=f1(i,j) + zfe(i,j)
	f2(i,j)=f1(i,j)
	1	    -fpdtjac(i,j)*( fpfluxe(i,j)-fpfluxe(i-1,j)
	1	    +fpfluxa(i,j)-fpfluxa(i,j-1) )
 
320	continue
 
325     continue
 
c
c       symmetrize barely trapped particles
c
	jjm=jp-njTrap
	jjp=jm+njTrap
	do 310 i=ibotto,maxen
	f2(i,jjm)=.5*( f2(i,jjm) + f2(i,jjp) )
310     f2(i,jjp)=f2(i,jjm)
 
C
C     MAKE OLD F NEW
C
	DO 350 J=1,maxxsi
	jget=j
	if(j .GT. jm .and. j .LT. jm+njTrap)jget=maxxsi+1-j
	DO 350 I=ibotto,maxen
350     f1(I,J)=f2(I,jget)*LOSS(I,J)
 
	if(LLwbnd .ne. 0)then
c--isotropize f at the origin for conservative B.C.:
	sum=0.0
	do 375 j=1,maxxsi
C@        sum=sum+f1(0,j)*qjaca(j)		  !CRAY
        sum=sum+(1.0D0*f1(0,j))*qjaca(j)	  !@VAX
375	CONTINUE
	averag=sum/coniso
	do 380 j=1,maxxsi
380     f1(0,j)=averag
 
	else
c--set df/dE=0 for other B.C.:
	do 385 j=1,maxxsi
385     f1(1,j)=f1(2,j)
	endif
 
C********************************************************************
C								    *
C       ADD IN SOURCE						    *
C								    *
C********************************************************************
 
 
1052	IF(TIME.GT.TOFF) GOTO 1060
 
	if(trise .le. 0.0) then
	do 1055 ien=0,maxen
	do 1055 ixi=1,maxxsi
1055	f1(ien,ixi)=f1(ien,ixi)+fnbsour(ien,ixi)
	else
	zrise=(1.-EXP(-(TIME+dtFPP/4.)/TRISE))
	do 1057 ien=0,maxen
	do 1057 ixi=1,maxxsi
1057	f1(ien,ixi)=f1(ien,ixi)+zrise*fnbsour(ien,ixi)
	endif
 
1060    continue
 
C********************************************************************
C								    *
C       PITCH SPACE IMPLICITLY				            *
C								    *
C********************************************************************
 
	TIME=TIME+dtFPP/2.
 
c--select B.C.:
1096    ibotto=1
	if(LLwbnd .EQ. 0)ibotto=2
 
C
C     ...D COEFFICIENTS
C
C     ......CTR
C
	DO 1901 I=ibotto,maxen
	  DA(I,1)=loss(i,1)*f1(i,1)/beta(i,1)
1901	CONTINUE
C
	M2=jm-njTrap
	DO 1100 J=2,M2
	  DO 1902 I=ibotto,maxen
	    DA(I,J)=loss(i,j)*
	1       (f1(I,J)-GAMA(I,J)*DA(I,J-1))
	1       /(BETA(I,J)+GAMA(I,J)*EA(I,J-1))
1902	  CONTINUE
1100    continue
C
C     ......CO
C
	DO 1903 I=ibotto,maxen
	  DA(I,maxxsi)=loss(i,maxxsi)*f1(i,maxxsi)/beta(i,maxxsi)
1903	CONTINUE
C
	M2=jp+njTrap
	do 1200 j = maxxsi-1, m2, -1
	  DO 1904 I=ibotto,maxen
	    DA(I,J)=loss(i,j)*(f1(I,J)-ALPHA(I,J)*DA(I,J+1))
	1       /(BETA(I,J)+ALPHA(I,J)*EA(I,J+1))
1904	  CONTINUE
1200    continue
C
C     ......TRAPPED
C
	DO 1905 I=ibotto,maxen
	  DA(I,jm)=loss(i,jm)*f1(I,jm)/(ALPHA(I,jm)+BETA(I,jm))
1905	CONTINUE
C
	M2=jp+1-njTrap
	do 1300 j = jm-1, m2, -1
	  DO 1906 I=ibotto,maxen
	    DA(I,J)=loss(i,j)*(f1(I,J)-ALPHA(I,J)*DA(I,J+1))
	1       /(BETA(I,J)+ALPHA(I,J)*EA(I,J+1))
1906	  CONTINUE
1300    continue
C
C     ...FIND F AT THE TRAPPING BOUNDARY
C
	J1=jp-njTrap
	J2=jm-njTrap
	J3=jp+1-njTrap
	J4=jp+njTrap
	DO 1907 I=ibotto,maxen
	  f2(I,J1)=(f1(I,J1)-ALPHA(I,J1)*DA(I,J3)
	1       -0.5*(GAMA(I,J1)*DA(I,J2) + gamap(i)*DA(I,j4)) )
	1       /(ALPHA(I,J1)*EA(I,J3)+BETA(I,J1)+
	1       0.5*(GAMA(I,J1)*EA(I,J2) + gamap(i)*EA(I,J4)) )
	1       *loss(i,j1)
1907	CONTINUE
C
C     ...FIND F EVERYWHERE
C
C     ......CTR
	M2=jm-njTrap
C
	do 1500 j= m2, 1, -1
	  DO 1908 I=ibotto,maxen
	    f2(I,J)=(EA(I,J)*f2(I,J+1)+DA(I,J))
1908	  CONTINUE
1500    continue
C
C     ......TRAPPED
C
	DO 1909 I = IBOTTO, MAXEN
	  f2(I,jm+njTrap)=f2(I,jp-njTrap)
1909	CONTINUE
C
	M2=jp+1-njTrap
	DO 1600 J=M2,jm
	  DO 1910 I=ibotto,maxen
	    f2(I,J)=(EA(I,J)*f2(I,J-1)+DA(I,J))
1910	  CONTINUE
1600    continue
C
C     ......CO
C
	M2=jp+njTrap
    	DO 1700 J=M2,maxxsi
    	  DO 1911 I=ibotto,maxen
	    f2(I,J)=(EA(I,J)*f2(I,J-1)+DA(I,J))
1911	  CONTINUE
1700    CONTINUE
C
C     MAKE OLD F NEW and subtract back out the explicit energy flux
c in preparation for the implicit energy step.
C
	DO 2000 J=1,maxxsi
	jget=j
	if(j .gt. jm .and. j .lt. jm+njTrap)jget=maxxsi+1-j
	DO 2010 I=ibotto,maxen
2010    f1(I,J)=f2(I,jget)
c2010    f1(I,J)=f2(I,jget)-zfe(i,jget)
c	if(LLwbnd .ne. 0)f1(0,j)=f1(0,j)-zfe(0,jget)
2000	continue
 
!c--isotropize f at the origin:
c	if(zfemult .ne. 0.0 .and. LLwbnd .ne. 0) then
c	sum=0.0
c	do 2075 j=1,maxxsi
cC@        sum=sum+f1(0,j)*qjaca(j)		  !CRAY
c        sum=sum+(1.0D0*f1(0,j))*qjaca(j)	  !@VAX
c2075	CONTINUE
c	averag=sum/coniso
c	do 2080 j=1,maxxsi
c2080	f1(0,j)=averag
c	endif
 
c--or set df/dE=0 near the origin:
	if(LLwbnd .EQ. 0)then
	do 2100 j=1,maxxsi
2100    f1(1,j)=f1(2,j)
	endif
 
C********************************************************************
C								    *
C       ADD IN SOURCE					            *
C								    *
C********************************************************************
 
4140    If(TIME.GT.TOFF) GOTO 4160
 
	if(trise .le. 0.0) then
	do 4155 ien=0,maxen
	do 4155 ixi=1,maxxsi
4155	f1(ien,ixi)=f1(ien,ixi)+fnbsour(ien,ixi)
	else
	zrise=(1.-EXP(-(TIME+dtFPP/4.)/TRISE))
	do 4157 ien=0,maxen
	do 4157 ixi=1,maxxsi
4157	f1(ien,ixi)=f1(ien,ixi)+zrise*fnbsour(ien,ixi)
	endif
 
4160    continue
 
! add in a source from cold neutrals which balance fast particle
! losses, since CX is a particle conserving process.  fsourc is
! calculated in MOMS:
	if(fsourc(0) .eq. 0.0)go to 4170
	do 4165 j=1,maxxsi
	do 4165 i=0,maxen
4165    f1(i,j)=f1(i,j)+fsourc(i)*loss(i,j)
 
C********************************************************************
C								    *
C       ENERGY SPACE IMPLICITLY				            *
C								    *
C********************************************************************
 
4170    TIME=TIME+dtFPP/2.
 
C
C     ...D COEFFICIENTS
C
 
	DO 4501 J = 1, maxxsi
	    de(j,maxen)=loss(maxen,j)*f1(maxen,j)/bete(maxen,j)
4501	CONTINUE
 
	if (LLwbnd .eq. 0)then
	    ibotto=2
	else
	    ibotto=1
	endif
 
	DO 4510 I=maxen-1,ibotto,-1
c--save some time by skipping the symmetric half of trapped space:
	  DO 4505 J = 1, JM
	      DE(J,I)=loss(i,j)*(f1(I,J)-alphe(I,J)*DE(J,I+1))
	1       /(BETE(I,J)+alphe(I,J)*EE(I+1,J))
4505	  CONTINUE
	  DO 4506 J = JM+NJTRAP+1, MAXXSI
	      DE(J,I)=loss(i,j)*(f1(I,J)-alphe(I,J)*DE(J,I+1))
	1       /(BETE(I,J)+alphe(I,J)*EE(I+1,J))
4506	  CONTINUE
4510	CONTINUE
c
c find f at the lower boundary:
c
	if(LLwbnd .ne. 0)then
 
c RF conservative BC's at i=0:
 
	zad=0.0
	zae=0.0
	zb=0.0
 
	do 4520 j=1,maxxsi
	jget=j
	if(j .gt. jm .and. j .le. jm+njtrap)jget=maxxsi+1-j
	zad=zad+(1.0d0*qjaca(j))*alphe(0,jget)*de(jget,1)
	zae=zae+(1.0d0*qjaca(j))*alphe(0,jget)*ee(1,jget)
4520	zb=zb+(1.0d0*qjaca(j))*bete(0,jget)
	zad=zad/coniso
	zae=zae/coniso
	zb=zb/coniso
 
	do 4522 j=1,maxxsi
4522	f2(0,j)=(f1(0,j)-zad)/(zae+zb)
 
	else
 
c NBI BC's at i=2:
 
	do 4525 j=1,jm
	f2(2,j)=de(j,2)/(1.0-EE(2,j))
4525	f2(1,j)=f2(2,j)
	do 4526 j=jm+njtrap+1,maxxsi
	f2(2,j)=de(j,2)/(1.0-EE(2,j))
4526	f2(1,j)=f2(2,j)
 
	endif
 
C
C     FIND F EVERYWHERE
C
 
	if (LLwbnd .eq. 0)then
	    ibotto=3
	else
	    ibotto=1
	endif
 
	do 4550 i = ibotto, maxen
	  DO 4530 J = 1, JM
	      f2(I,J)=(EE(I,J)*f2(I-1,J)+DE(J,I))
4530	  CONTINUE
	  DO 4535 J = JM+NJTRAP+1, MAXXSI
	      f2(I,J)=(EE(I,J)*f2(I-1,J)+DE(J,I))
4535	  CONTINUE
4550	CONTINUE
 
C
C     MAKE OLD F NEW -- INCLUDE FISHBONE LOSSES:
C
	if (LLwbnd .eq. 0)then
	    ibotto=1
	else
	    ibotto=0
	endif
 
	DO 4600 J=1,maxxsi
	jget=j
	if(j .gt. jm .and. j .LE. jm+njTrap)jget=maxxsi+1-j
	DO 4600 I=ibotto,maxen
4600    f1(i,j)=f2(i,jget)*fishl(i,j)
 
c--if f<0 print out an error message.  (Only do this the first
c       maxerr times.)  Then set f=0.  Note that f<0 does not always
c       mean a numerical instability has set in.  Near the xsi=+/-1
c       boundaries, the RF operator is supposed to drive f to zero,
c       since it give particles a kick in vper.  Sometimes it just
c       slightly overshoots zero.
	data maxerr/maxen/
 
!temporarily skip this
	data lforcf/0/
4820    if(lforcf .EQ. 0)return
 
4849    do 4900 j=1,maxxsi
	do 4900 i=ibotto,maxen
	if(f1(i,j) .GE. 0.0)goto 4900
	nf0err=nf0err+1
	if(nf0err .LE. maxerr)write(lsysout,4850) time,i,j,f1(i,j)
4850    format(' f<0 at time=',f10.4,' f(',i4,',',i4,')=',e10.4)
	f1(i,j)=0.0
4900    continue
 
	return
	end
