	subroutine SCCF
c
c This subroutine calculates a number of different bounce-averaging
c integrals, using the zero-banana-width approximation, for each
c different pitch angle.  The integrals which are calculated are
c listed below:
c
c btime	  (Integral dl/abs(v_parrallel))/(2 pi q Rmaj/v) normed bounce time
c vp2av   <vpar**2/v**2>          to get beam perp and par betas
c bratio  <Bmin/B>-1       needed for pitch angle scattering operator
c vparav  <Vpar/V>
c rvparav <R Vpar/V>
c rmaj2av <R**2>
c vparvr  <Vpar/V/R>
c
c vprec   toroidal transit frequency (sec**-1) for trapped and passing ions
c	    (added by Rob Goldston, 1/29/83)
c
c dVolnorm = (2 pi q Rmaj / Bmin Integral dl/B)
c
c Additional quantities calculated by this subroutine:
c njtrap
c
c Most of the specifics about the plasma geometry come into the problem
c through the bounce-averages.  For general axisymmetric toroidal geometry,
c the bounce time can be written as:
c
c Tau_bounce = Integral dl/abs(v_par)
c	     = Integral d(theta) J B / abs(v_par)
c
c With the proper choice of the poloidal angle theta, the jacobian
c J can be written as J=(q R**2 / It), where It= (R B_tor) is a flux
c surface quantity, and q is the inverse rotational transform and is
c also a flux surface quantity.
c
c However, FPP and TRANSP do not use this choice of poloidal angle,
c so the Jacobian must be explictly evaluated.  This is done in
c MOMRY and FPGEO.
c
c--Tau bounce is defined by gwh for circulating particles as
c	the time to make one complete orbit to return to the
c	same poloidal angle, and for trapped particles it is
c	the time it takes to make one half orbit from one banana
c	tip to the other.  For vpar>>vperp, the passing particle
c	bounce time reduces to (2 pi q Rmaj)/vpar.
c
c--SCCF calculates BTIME,which is related to the bounce time by:
c		tau bounce = (2 pi q Rmaj / v) * btime
c
c--while I am on the subject, I might as well define other
c normalizations. f_fpp used in this code has the units
c of particles per (cm)**3 (eV)**1.5, and is related
c to the velocity space distribution function f by
c
c	f_fpp = (2**.5 2 pi / 25) (erg / (m eV))**1.5 f
c
c where m is the particle mass in grams.
c f has units of particles per (cm)**3 (cm/sec)**3, and is
c normalized such that the local particle density is given
c by (in TeX notation):
c
c	n = \int d^3 v f
c
c--so the average number of particles on a flux surface is:
c	<n> = 25 dVolnorm (sum over xsi of) (sum over energy of)
c		dxsi dE sqrt(E) abs(xsi) btime f_fpp
C
c where dVolnorm = (2 pi q Rmaj / Bmin Integral dl/B)
C
 
c
c large aspect ratio concentric circular flux surface definitions:
c
c	VPREC == <[VPAR**2/V**2 + VPER**2/(2*V**2)]
c	* [Q*CTH/rOverR + STH*(TH-PI)*r*dq/dr/rOverR]>	(UNTRAPPED PTCLS)
c	* [Q*CTH/rOverR + STH*TH*r*dq/dr/rOverR]>  (TRAPPED PTCLS)
c
c Note that the definition of the precession speed for untrapped
c particles is probably not well defined.  Because of the shear in
c the magnetic field (dq/dr), the measure of the precession of a particle
c relative to a field line depends on which flux surface you choose the
c reference field line from.  It may depend on the type of mode you
c are interacting with, i.e., ballooning modes are localized to the
c large R side of the plasma, while kink modes are more constant,
c and barely passing ions spend most of their time on the small R side
c of the plasma.  The choice made here (TH-PI) is perhaps best for
c strongly passing particles and ballooning-type modes, while Rob's
c original choice (PI-TH) is perhaps best for looking at barely
c passing particles and flute modes (like the m=1 kink usually
c driving the fishbone instability).
c
 
	include 'FPPCOM'
 
C
C     CALCULATE GEOMETRIC AND MAGNETIC PARAMETERS
C
	zpi=abs(acos(-1.0))
 
c this algorithm for obtaining the max and min of B on a flux
c surface won't work for heavily indented or extremely high beta
c plasmas (i.e., assumes B=Btor*Rmaj/R):
	zth=0.0
	zlam=0.0
     	call fpgeo(zth,zlam,zRlocal,zJacB,zBmin,zthdot,zvpar)
	zth=zpi
	call fpgeo(zth,zlam,zRlocal,zJacB,zBmax,zthdot,zvpar)
 
c--pitch angle of barely trapped particles:
	ztrans=sqrt(1.0-zBmin/zBmax)
 
c--find the index of barely trapped particles
c-- define njTrap so that xsi(maxxsi/2+njTrap) <= ztrans:
	jp=maxxsi/2+1
20	do 30 j=jp,maxxsi
	if(ztrans .gt. xsi(j+1))goto 30
	njTrap=j
	go to 40
30	continue
	njTrap=maxxsi
40	njTrap=njTrap-maxxsi/2
	if(njTrap .lt. 3 .or. njTrap .ge. maxxsi/2-1) then
	write(lsysout,*)
	1   'SCCF warning: only have njtrap =',njtrap,' grid points'
	write(lsysout,*)
	1   ' to resolve the trapped region of velocity space for'
	write(lsysout,*)
	1   ' nshe=',nshe,' where the trapped boundary is at'
	write(lsysout,*) ' vpar0/v = ',ztrans
	endif
 
	if(njTrap .eq. maxxsi/2) then
	    call aborter(lsysout,
	1	'SCCF:  njTrap covers all velocity space')
	endif
c
c use second-order Runge-Kutta formula to integrate along particle
c orbits to find the bounce-time, and other bounce-averaged quantities.
c
 
	do 2500 j=maxxsi/2+1,maxxsi
 
 
c--for the grid point which represents barely trapped particles, we
c	need the average value of the bounce time over the region
c	represented by that grid point.	 Assuming that the bounce
c	time is logarithmically singular, we can approximate this by
c	tau bounce half a grid spacing away:
 
	data zsing /0.5/
	zxsi=xsi(j)
	if(j .eq. maxxsi/2+njTrap)then
		zdeltx=xsi(maxxsi/2+njTrap+1)-xsi(maxxsi/2+njTrap)
		zxsi=ztrans-zdeltx*zsing
	if(njTrap .eq. 1) zxsi=xsi(j)/2
	endif
 
	if(j .eq. maxxsi/2+njTrap+1)then
		zdeltx=xsi(maxxsi/2+njTrap+1)-xsi(maxxsi/2+njTrap)
		zxsi=ztrans+zdeltx*zsing
	endif
 
	zlam=(1-zxsi**2)/zBmin
	zBend=zBmax
	if(zlam .gt. 0.0) zBend=1/zlam
	if(zBend .gt. zBmax)zBend=zBmax
c find the maximum theta of the orbit:
	call fpgeoinv(zBend,zthend)
 
c
c Integrate using a second-order Runge-Kutta integrator.
c Integrate around orbit taking constant steps in time (dt=dl/vpar)
c rather than constant theta steps.  This automatically puts more
c steps near the banana tip.  The 2cd order RK integrator provides
c a good compromise between accuracy, speed, and simplicity (i.e.,
c no need to evaluate higher order derivatives).  For this 2cd order
c method, the relative error of all of the bounce averages will scale
c as the stepsize squared.  The absolute error may still get large near
c the singularity at the trapped passing-boundary, but this is only
c a fairly weak logarithmic singularity.
c We are integrating toward the banana tip, so take care to not
c evaluate 1/vpar at vpar=0.0.
c
c For the bounce-time equation, the second order Runge-Kutta method is:
c
c d(theta)/dt = abs(vparallel)/(Jac*B)
c	    = v(theta)
c
c theta1=theta0+0.5*dt*v(theta0)
c theta2=theta0+dt*v(theta1)
c
c For the other quantities to be bounce-averaged, since we already know
c theta1 from above, the second order Runge-Kutta method only needs
c to evaluate one function:
c
c dF/dt = f(theta)
c
c F(theta2)=F(theta0)+dt*f(theta1)
c
c
c
c set up initial conditions:
	zth0=0.0
	ztime=0.0
	zbinvsum=0.0
	zpsum=0.0
	zrvpsum=0.0
	zvpsum=0.0
	zvpvrsum=0.0
	zr2sum=0.0
	zbsum=0.0
 
	call fpgeo(zth0,zlam,zRlocal,zJacB,zBlocal,zthdot,zvpar)
 
c zerrrel should be set to the same value in fpgeoinv:
	data zerrrel /5.e-5/
	if (zerrrel .eq. 0.0)zerrrel=1.0/maxxsi**2
	zdt=sqrt(zerrrel)*zthend/zthdot
 
c begin integral around the orbit:
	ilast=0
2400	zth1=zth0+0.5*zdt*zthdot
 
	if(zth1 .gt. zthend)then
	    ilast=1
	    zth1=(zthend+zth0)/2.0
	    call fpgeo(zth1,zlam,zRlocal,zJacB,zBlocal,zthdot,zvpar)
	    if(zthdot .eq. 0.0)goto 2480
	    zdt=(zthend-zth0)/zthdot
	    goto 2450
	endif
 
	call fpgeo(zth1,zlam,zRlocal,zJacB,zBlocal,zthdot,zvpar)
	zth2=zth0+zdt*zthdot
 
c take care to hit endpoint exactly:
	if(zth2 .gt. zthend)then
	    ilast=1
	    zth1=(zthend+zth0)/2.0
	    call fpgeo(zth1,zlam,zRlocal,zJacB,zBlocal,zthdot,zvpar)
	    if(zthdot .eq. 0.0)goto 2480
	    zdt=(zthend-zth0)/zthdot
	    goto 2450
	endif
 
c*********************************************************************
c accumulate sums:
 
2450	ztime=ztime+zdt
 
c Bmin/B:
	zbinvsum=zbinvsum+zdt*(zBmin/zBlocal-1.0)
 
c B:
	zBsum=zBsum+zdt*zBlocal
 
c R (vpar/v) (Bphi/B) in the Bphi=B approximation:
	zrvpsum=zrvpsum+zdt*zRlocal*zvpar
 
c vpar/v:
	zvpsum=zvpsum+zdt*zvpar
 
c vpar/v/R:
	zvpvrsum=zvpvrsum+zdt*zvpar/zRlocal
 
c R*R:
	zr2sum=zr2sum+zdt*zRLocal*zRlocal
 
c while most of the above formulas are exact, this expression for the
c precession speed makes the assumptions Bp<<Bt and B=Bt=Btor*Rmaj/R:
 
	call fpgeogrd(zth1,zdbpsi,zdbtheta,zdlqpsi)
	if(j .le. maxxsi+njtrap)then
	zprec=0.5*(1.0+1.0-zlam*zBlocal)*Rmaj*zRlocal*
	1   (-zdbpsi+zdbtheta*zdlqpsi*zth1)
	else
	zprec=0.5*(1.0+1.0-zlam*zBlocal)*Rmaj*zRlocal*
	1   (-zdbpsi+zdbtheta*zdlqpsi*(zth1-zpi))
	endif
 
	zpsum=zpsum+zdt*zprec
 
c*********************************************************************
 
c declare we reached the end if we hit the end exactly (ilast=1)
c or we aren't moving anymore:
	if(ilast .ne. 0 .or. zth2 .eq. zth0)goto 2480
 
c setup for next orbit step:
	zth0=zth2
	call fpgeo(zth0,zlam,zRlocal,zJacB,zBlocal,zthdot,zvpar)
	if(zthdot .eq. 0.0)goto 2480
	goto 2400
 
c end of orbit integral:
2480	continue
	btime(j)=ztime*2.0
	bratio(j)=zbinvsum/ztime
	if(j .eq. maxxsi) fpIdlB=zBsum
	vp2av(j)=1.0-zlam*zBsum/ztime
	rvparav(j)=zrvpsum/ztime
	vparav(j)=zvpsum/ztime
	vparvr(j)=zvpvrsum/ztime
	rmaj2av(j)=zr2sum/ztime
2500	vprec(j)=zpsum/ztime
 
c <vpar/v> is zero for trapped particles:
	do 2515 j=1,njtrap
	rvparav(maxxsi/2+j)=0.
	vparvr(j)=0.0
2515	vparav(maxxsi/2+j)=0.
 
c normalize various integrals:
	do 2520 j=maxxsi/2+1,maxxsi
2520	btime(j)=btime(j)/(2*zpi*qpro(nshe)*Rmaj)
 
c fill up arrays for positive and negative xsi:
	do 2530 j=1,maxxsi/2
	bratio(j)=bratio(maxxsi+1-j)
	vprec(j)=vprec(maxxsi+1-j)
	vp2av(j)=vp2av(maxxsi+1-j)
	rvparav(j)=-rvparav(maxxsi+1-j)
	vparav(j)=-vparav(maxxsi+1-j)
	vparvr(j)=-vparvr(maxxsi+1-j)
	rmaj2av(j)=rmaj2av(maxxsi+1-j)
2530	btime(j)=btime(maxxsi+1-j)
 
c a few integrals needed for volume and area averages:
	dvolnorm=1./((bratio(maxxsi)+1.0)*btime(maxxsi))
c Integral dl B0:
	fpIdlB0=zBmin*btime(maxxsi)*2*zpi*qpro(nshe)*Rmaj
c Integral dl B0/B:
	fpIdlB0B=(bratio(maxxsi)+1.0)*fpIdlB0/zBmin
c Integral dl B:
	fpIdlB=fpIdlB*2.0
 
c compare numerical bounce time with the analytic value in the
c deeply trapped limit:
 
	call fpgeo(0.0,0.0,zRlocal,zJacB,zB0,zthdot,zvpar)
	zbtime0=zJacB*sqrt(zB0)*zpi
	ztheta=0.2
	call fpgeo(ztheta,0.0,zRlocal,zJacB,zB1,zthdot,zvpar)
	zbtime0=zbtime0/(sqrt(zB1-zB0)/ztheta)
	zbtime0=zbtime0/(2*zpi*qpro(nshe)*Rmaj)  ! normalize
	if( abs((zbtime0-btime(maxxsi/2+1))/zbtime0) .gt. 0.1)
	1   call aborter(lsysout,'SCCF integration inaccurate!')
c (or the evaluation of d**2 B/ d theta**2 for the analytic formula
c is inaccurate).
 
c save btime(xsi,r) for future use in rdstepdw:
	do 2540 j=1,maxxsi
	btxsi(j,nshe)=btime(j)
2540	continue
 
C
C     DISPLAY TAU BOUNCE, <BMIN/B> - 1, <VPAR**2/V**2>, AND VPREC
C
	data lplotb/0/ !1 = plot bounce averages vs. pitch angle
	if(lplotb .eq. 0) goto 777
	jstart=maxxsi/2+1
 
	call initt(120) !erase screen
	call binitt
	call fpgrlab
	call npts(maxxsi/2)
	call check(xsi(jstart),btime(jstart))
	call dsplay(xsi(jstart),btime(jstart))
	write(text1,419) nshe
419	format(' (Bounce Time)/(2 pi q Rmaj/v)  on radial shell ',
	1   i2,' $')
	call agtcgt(750,text1)
	call agtchs('Vpar/V @ Bmin$')
	WRITE(TEXT1,420) zbtime0, ztrans
420	FORMAT('Trapped limit btime=',f5.3,
	1   '   Boundary Vpar/V=',F5.3,' $')
	CALL AGTCGT(0,TEXT1)
	CALL UPAUSE(0)
 
	call initt(120) !erase screen
	call binitt
	call fpgrlab
	call npts(maxxsi/2)
	call check(xsi(jstart),bratio(jstart))
	call dsplay(xsi(jstart),bratio(jstart))
	call agtcgt(750,'<BMIN/B>-1$')
	call agtchs('Vpar/V @ Bmin$')
c--bmbm is exact bratio in the limit vpar >> vperp for concentric circles:
	zBMBM=1./(1.+rminpro(nshe)/Rmaj)-1.0
c--exact bratio=<Bmin/B>-1 of barely trapped particles = -ztrans**2:
	zavmim=zBmin/zBmax-1.0
	WRITE(TEXT1,450) zAVMIM, zBMBM
450	FORMAT('BOUNDARY VALUE=',F5.3,2X,'Circular PAR. LIMIT=',F5.3,'$')
	CALL AGTCGT(0,TEXT1)
	CALL UPAUSE(0)
 
	call initt(120) !erase screen
	call binitt
	call fpgrlab
	call npts(maxxsi/2)
	call check(xsi(jstart),vp2av(jstart))
	call dsplay(xsi(jstart),vp2av(jstart))
	call agtcgt(750,'<Vpar**2/V**2>$')
	call agtchs('Vpar/V @ Bmin$')
	CALL UPAUSE(0)
 
	call initt(120) !erase screen
	call binitt
	call fpgrlab
	call npts(maxxsi/2)
	call check(xsi(jstart),vprec(jstart))
	call dsplay(xsi(jstart),vprec(jstart))
	call agtcgt(750,
	1   'Precession rate <dPhi/dt> / (v**2/(Omega R*R)@Rmaj) $')
	call agtchs('Vpar/V @ Bmin$')
	zprecn1=qpro(nshe)/(rminpro(nshe)/Rmaj)/2.0
	zprecn2=Rmaj*dqdrpro(nshe)
	WRITE(TEXT1,480) zprecn1, zPRECN2
480	FORMAT('Circular TRAPPED LIMIT =', F6.2,'   PASSING LIMIT =',
     >	F6.2,'$')
	CALL AGTCGT(0,TEXT1)
	CALL UPAUSE(0)
 
	call initt(120) !erase screen
	call binitt
	call fpgrlab
	call npts(maxxsi/2)
	call check(xsi(jstart),vparav(jstart))
	call dsplay(xsi(jstart),vparav(jstart))
	call agtcgt(750,'<Vpar/V>$')
	call agtchs('Vpar/V @ Bmin$')
	CALL UPAUSE(0)
 
	call initt(120) !erase screen
	call binitt
	call fpgrlab
	call npts(maxxsi/2)
	call check(xsi(jstart),RVparav(jstart))
	call dsplay(xsi(jstart),RVparav(jstart))
	call agtcgt(750,'<R Vpar/V>$')
	call agtchs('Vpar/V @ Bmin$')
	CALL UPAUSE(0)
 
	call finitt(0,0) !return to text screen
  777 CONTINUE
 
 
	return
	end
